@@ -112,6 +112,9 @@ soup_queue_error_cb (gboolean body_started, gpointer user_data)
 		soup_message_issue_callback (req);
 		break;
 	}
+
+	if (req->connection)
+		soup_connection_set_used (req->connection);
 }
 
 static SoupTransferDone
@@ -247,6 +250,8 @@ soup_queue_read_done_cb (const SoupDataBuffer *data,
 {
 	SoupMessage *req = user_data;
 
+	soup_connection_set_used (req->connection);
+
 	req->response.owner = data->owner;
 	req->response.length = data->length;
 	req->response.body = data->body;
@@ -604,6 +609,13 @@ proxy_connect (SoupContext *ctx, SoupMessage *req, SoupConnection *conn)
 				"Unable to create secure data "
 				"tunnel through proxy");
 
+			/*
+			 * If the tunnelling failed, our connection will
+			 * have been freed by the requeue in
+			 * proxy_https_connect()
+			 */
+			req->connection = NULL;
+
 			soup_message_issue_callback (req);
 			return TRUE;
 		}
